/**
 * BlobDetection Class
 * Every instance cantracks its own color and luminosity
 */

class PuppetBlobTracker{
  public int width, height;
  private int maskColor;                  //color to mask the input image by
  private float luminosityThreshold;      //how much the lum can vary
  private color averageColor;             //an average color to track the hue (might need to be just a number)
  private float colorDistance;            //the distance in the color cube
  private BlobDetection blobDetection;    //the blob detection for luminosity
  private PImage inputImg;                //A 80X60 image to work on

  private boolean bDrawBlobs;              //True if fancy blobs need to be drawn
  private color lineColor;

  /**
   * Constructor
   */
  public PuppetBlobTracker(int iWidth, int iHeight){
    this.width = iWidth;
    this.height = iHeight;
    
    this.maskColor = 0xFF000000;      //set to black
    this.luminosityThreshold = 0.1;

    this.averageColor = color(0); //black
    this.colorDistance = 50;

    this.blobDetection = new BlobDetection(this.width, this.height);
    this.bDrawBlobs = false;      
  }

  /**
   * Tick Method
   * We need to filter the image and pass it to the blob filter
   */
  public void tick(float delta){
    //get the new filter img to detec blocs from
    PImage filterImg = this.filterImage();

    //now do the blob tracking
    this.blobDetection.setThreshold(this.luminosityThreshold);
    this.blobDetection.computeBlobs(filterImg.pixels);


  }

  /**
   * Draw
   * Draw lines and such if the flags say so!
   */
  public void draw(){
    if(this.bDrawBlobs){
      this.drawBlobsAndEdges(true, true);
    }
  }

  /**
   * Creates a filtered image to use for the blob tracker
   */
  public PImage filterImage(){
    float disty;
    PImage filteredImg = new PImage(this.inputImg.width, this.inputImg.height);
    int nbPix = this.inputImg.width*this.inputImg.height;

    for (int i=0 ; i<nbPix ; i++)
    {
      disty = dist(maskColor >> 16 & 0xFF, maskColor >> 8 & 0xFF, maskColor & 0xFF, 
      this.inputImg.pixels[i] >> 16 & 0xFF, this.inputImg.pixels[i] >> 8 & 0xFF, this.inputImg.pixels[i] & 0xFF);

      if(disty > colorDistance){
        filteredImg.pixels[i] = color(0);
      } 
      else {
        filteredImg.pixels[i] = (this.inputImg.pixels[i] & this.maskColor);
      }
    }
    //return the img ready to be processed
    return filteredImg;
  }

  /**
   *Draw the Blobs on the Image
   */
  void drawBlobsAndEdges(boolean drawBlobs, boolean drawEdges){
    noFill();
    Blob b;
    EdgeVertex eA,eB;
    
        

    for (int n=0; n < blobDetection.getBlobNb() ; n++)
    {
      //println("Drawing a couple edges, oi!");
      b=blobDetection.getBlob(n);
      if (b!=null){
        // Edges
        if (drawEdges){
          strokeWeight(2);
          stroke(this.maskColor);
          for (int m=0;m<b.getEdgeNb();m++){
            eA = b.getEdgeVertexA(m);
            eB = b.getEdgeVertexB(m);
            if (eA !=null && eB !=null)
              line(eA.x*width, eA.y*height, eB.x*width, eB.y*height);
          }
        }
        
        // Blobs
        if (drawBlobs){
          strokeWeight(1);
          stroke(this.maskColor);
          rect(b.xMin * width, b.yMin * height, b.w * width, b.h * height);
        }
      }
    }
  }

  /**
   * Finds Largest Blob
   */
  Blob findLargestBlob(){
    Blob b, largestBlob=null;
    float largestBlobSize = 0;
    for (int n = 0; n < blobDetection.getBlobNb(); n++){
      b=blobDetection.getBlob(n);
      if(b != null){
        if((b.xMin * width) * (b.yMin * height) > largestBlobSize) {
          largestBlob=b;
          largestBlobSize = (b.xMin * width) * (b.yMin * height);
        }   
      } 
    }
    return largestBlob;
  }

  /**
   * Get the the centroid
   */
  float getAverageCentroid(char c){
    Blob b;
    float averageX = 0; 
    float averageY = 0;
    
    if(c == 'x'){
      for (int n = 0; n < blobDetection.getBlobNb(); n++){
        b=blobDetection.getBlob(n);
        if(b != null) averageX += ((b.xMin * width) - width/2 )/(width/2);
      }
      return averageX / blobDetection.getBlobNb();  
    }
    
    
    if(c == 'y'){
      for (int n = 0; n < blobDetection.getBlobNb(); n++){
        b=blobDetection.getBlob(n);
        if(b != null) averageY += ((b.yMin * height) - height/2 )/(height/2);
      }
      return averageY / blobDetection.getBlobNb();  
    }
    return 0.0;
  }


/////////////////////////
// GETTERS AND SETTERS //
/////////////////////////

public void setMaskColor(int col){
  this.maskColor = col;
  }

public int getMaskColor(){
  return this.maskColor;
  }

public void setLuminosityThreshold(float thresh){
  this.luminosityThreshold = thresh;
  }

public float getLuminosityThreshold(){
  return this.luminosityThreshold;
  }

public void setAverageColor(color col){
  this.averageColor = col;
  }
public color getAverageColor(){
  return this.averageColor;
  }

public void setColorDistance(float d){
  this.colorDistance = d;
  }
public float getColorDistance(){
  return this.colorDistance;
  }

public void setInputImg(PImage img){
  this.inputImg = img;
  }

public void setDrawBlobs(boolean b){
  this.bDrawBlobs = b;
  }

public boolean getDrawBlobs(){
  return this.bDrawBlobs;
  }   



}
